/**********************************************************************
 *<
	FILE: FloatSpringController.h

	DESCRIPTION:	Includes for Plugins

	CREATED BY:

	HISTORY:

 *>	Copyright (c) 2000, All Rights Reserved.
 **********************************************************************/

#ifndef __FLOATSPRINGCONTROLLER__H
#define __FLOATSPRINGCONTROLLER__H

#include "Max.h"
#include "resource.h"
#include "istdplug.h"
#include "iparamb2.h"
#include "iparamm2.h"



extern TCHAR *GetString(int id);

extern HINSTANCE hInstance;

#define FLOATSPRINGCONTROLLER_CLASS_ID	Class_ID(0x19738a91, 0x15cf8850)




#define PBLOCK_REF	0



enum { floatspringcontroller_params };


//TODO: Add enums for various parameters
enum { 
	pb_mass,pb_strength,pb_dampening,pb_samples,pb_driver,pb_cache,pb_enable,pb_manualsolve
};


class FloatSpringControlDlgProc;


class FloatSpringController : public StdControl {
	public:


		// Parameter block
		IParamBlock2	*pblock;	//ref 0
		IParamMap2		*pDialogMap;


		
		// Loading/Saving
		IOResult Load(ILoad *iload);
		IOResult Save(ISave *isave);

		//From Animatable
		Class_ID ClassID() {return FLOATSPRINGCONTROLLER_CLASS_ID;}		
		SClass_ID SuperClassID() { return CTRL_FLOAT_CLASS_ID; }
		void GetClassName(TSTR& s) {s = GetString(IDS_CLASS_NAME);}

		RefTargetHandle Clone( RemapDir &remap );
		RefResult NotifyRefChanged(Interval changeInt, RefTargetHandle hTarget, 
			PartID& partID,  RefMessage message);


		int NumSubs() { return 1; }
		TSTR SubAnimName(int i) { return GetString(IDS_PARAMS); }				
		Animatable* SubAnim(int i) { return pblock; }

		// TODO: Maintain the number or references here
		int NumRefs() { return 1; }
		RefTargetHandle GetReference(int i) { return pblock; }
		void SetReference(int i, RefTargetHandle rtarg) { pblock=(IParamBlock2*)rtarg; }




		int	NumParamBlocks() { return 1; }					// return number of ParamBlocks in this instance
		IParamBlock2* GetParamBlock(int i) { return pblock; } // return i'th ParamBlock
		IParamBlock2* GetParamBlockByID(BlockID id) { return (pblock->ID() == id) ? pblock : NULL; } // return id'd ParamBlock

		void DeleteThis() { delete this; }		
		//Constructor/Destructor

		FloatSpringController();
		~FloatSpringController();		

		// Animatable methods		
		int IsKeyable() {return 1;}		
		BOOL IsAnimated() {return TRUE;}

		void EditTrackParams(
			TimeValue t,
			ParamDimensionBase *dim,
			TCHAR *pname,
			HWND hParent,
			IObjParam *ip,
			DWORD flags);
		
		int TrackParamsType() {return TRACKPARAMS_WHOLE;}

		// Control methods				
		void Copy(Control *from);
		BOOL IsLeaf() {return TRUE;}
		void CommitValue(TimeValue t) {}
		void RestoreValue(TimeValue t) {}

		void HoldRange();
		Interval GetTimeRange(DWORD flags) {return range;}
		void EditTimeRange(Interval range,DWORD flags);
		void MapKeys(TimeMap *map,DWORD flags );


		Interval range;
		
		FloatSpringControlDlgProc *proc;
		BOOL activeDialog;

		HWND hWnd;


		// Control methods


		// StdControl methods
		void GetValueLocalTime(TimeValue t, void *val, Interval &valid, GetSetMethod method=CTRL_ABSOLUTE);
		void SetValueLocalTime(TimeValue t, void *val, int commit, GetSetMethod method);		
		void Extrapolate(Interval range,TimeValue t,void *val,Interval &valid,int type);		
		void *CreateTempValue() {return new float;}
		void DeleteTempValue(void *val) {delete (float*)val;}
		void ApplyValue(void *val, void *delta) {*((float*)val) += *((float*)delta);}
		void MultiplyValue(void *val, float m) {*((float*)val) *= m;}
		void RebuildCache();

		BOOL stopRef;
		BOOL rebuildCache;

};


class FloatSpringControllerClassDesc:public ClassDesc2 {
	public:
	int 			IsPublic() { return TRUE; }
	void *			Create(BOOL loading = FALSE) { return new FloatSpringController(); }
	const TCHAR *	ClassName() { return GetString(IDS_CLASS_NAME); }
	SClass_ID		SuperClassID() { return CTRL_FLOAT_CLASS_ID; }
	Class_ID		ClassID() { return FLOATSPRINGCONTROLLER_CLASS_ID; }
	const TCHAR* 	Category() { return GetString(IDS_CATEGORY); }

	const TCHAR*	InternalName() { return _T("FloatSpringController"); }	// returns fixed parsable name (scripter-visible name)
	HINSTANCE		HInstance() { return hInstance; }				// returns owning module handle

};



class FloatSpringControlDlgProc: public ParamMap2UserDlgProc 
{
	public:
		FloatSpringController *cont;
		BOOL DlgProc(TimeValue t, IParamMap2 *map, HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
		void DeleteThis()  {delete this;}
		void SetThing(ReferenceTarget *m) { cont = (FloatSpringController *)m;}
};


#endif // __FLOATSPRINGCONTROLLER__H
